<?php

class MahnRot_API
{
    private $username = false;
    private $password = false;
    private $userid = false;
    
    function __construct($username, $password)
    {
        $this->username = $username;
        $this->password = $password;
        
        if($this->verifyCredentials() === false)
        {
            throw new Exception("Invalid Credentials");
        }
    }
    
    public function verifyCredentials()
    {
        $url = "https://".urlencode($this->username).":".urlencode($this->password)."@kundenverwaltung.mahn-rot.de/api/index.php?action=verify_credentials";
        $result = file_get_contents($url, false);
        if(!$result)
        {
            return false;
        }
        
        $json = json_decode($result, true);
        if($json['status'] == 'ok')
        {
            $this->userid = $json['userid'];
            return $this->userid;
        }
        
        return false;
    }
    
    public function submitAkte(MahnRot_Akte $akte)
    {
        $plugin_billing_ueber_basiszinssatz = 5; //privatkunde
        if($akte->getSchuldner()->getSchuldnerType() == MahnRot_SchuldnerType::COMPANY)
        {
            $plugin_billing_ueber_basiszinssatz = 9; //unternehmer
        }

        $plugin_billing_anwalt_hebesatz = 0.9; //standard??
        
        //precheck
        $forderung_total = 0;
        $forderungen = $akte->getForderungen();
        
        if (!is_array($forderungen)) {
            error_log("[MAHN ROT API] submitAkte: \$forderungen was not an array. Converting to empty array.");
            $forderungen = array();
        }
        foreach($forderungen as $forderung)
        {
            $forderung_total += $forderung->getTotalPrice($cents=false);
        }
        if($forderung_total <= 0)
        {
            throw new Exception("Forderung ungueltig");
        }
        $schuldner = $akte->getSchuldner();
        $new_schuldner_id = $this->_submitCustomer($schuldner, $plugin_billing_ueber_basiszinssatz, $plugin_billing_anwalt_hebesatz);
        $this->_submitForderungen($new_schuldner_id, $akte->getForderungen(), $forderung_total, $schuldner, $plugin_billing_ueber_basiszinssatz);
        
        return $new_schuldner_id;
    }
    
    private function _submitCustomer($schuldner, $plugin_billing_ueber_basiszinssatz, $plugin_billing_anwalt_hebesatz)
    {
        $schuldnerdata = $schuldner->getSchuldnerData();
        //create customer
        $post_data = array(
            'kundennummer' => '1234',
            'email' => $schuldnerdata['email'],
            'firma' => $schuldnerdata["firma"],
            'vorname' => $schuldnerdata["vorname"],
            'nachname' => $schuldnerdata["nachname"],
            'anrede' => $schuldnerdata['anrede'], //Frau|Mann|Divers
            'str' => $schuldnerdata["str"],
            'hausnr' => $schuldnerdata["hausnr"],
            'ort' => $schuldnerdata["ort"],
            'plz' => $schuldnerdata["plz"],
            'land' => $schuldnerdata['land'],
            'tel' => $schuldnerdata['tel'],
            'mobile' => $schuldnerdata['mobile'] ?? '',
            'fax'    => $schuldnerdata['fax']    ?? '',
            'notes' => 'Test bla',
            'iban' => '',
            'bic' => '',
            'vertragsgegenstand' => 'Testvertrag',
            'vertragsdatum' => '01.01.2025',
            'schuldnertype' => $schuldner->getSchuldnerType(),
        );
        $post_data = http_build_query($post_data);
        $context_options = array(
                'ssl'=>array(
                  'verify_peer'=>false,
                  'verify_peer_name'=>false,
                  'allow_self_signed'=>true,
                ),
                'http'=>
                array(
                    'timeout' => 5,
                )
              );
        $context_options['http'] = array (
                    'method' => 'POST',
                    'header'=> "Content-type: application/x-www-form-urlencoded\r\n",
                    'content' => $post_data
                    );
        $context = stream_context_create($context_options);
        $json_data = file_get_contents("https://".urlencode($this->username).":".urlencode($this->password)."@kundenverwaltung.mahn-rot.de/api/index.php?action=addcustomer", false, $context);
        $retdata_billing = json_decode($json_data, true);
        if(!$retdata_billing)
        {
            throw new Exception("Fehler beim speichern im MAHN & ROT System!");
            return false;
        }
        
        return $retdata_billing['new_schuldner_id'];
    }
    
    private function _submitForderungen($new_schuldner_id, $forderungen, $forderung_total, $schuldner, $plugin_billing_ueber_basiszinssatz)
    {
        if (!is_array($forderungen)) {
            error_log("[MAHN ROT API] _submitForderungen: \$forderungen was null or not array. Converting to array.");
            $forderungen = array();
        }
        
        $forderung_total = 0;
        foreach($forderungen as $forderung)
        {
            $forderung_total += $forderung->getTotalPrice($cents=false);
        }
        $field_match_values = array();
        $field_match_values['KUNDENNUMMER'] = $schuldner->getSchuldnerData()['kundennummer'];

        $field_match_values['SUMME_OFFENE_POSTEN'] = abs($forderung_total);
        $field_match_values['SUMME_BIS_VERTRAGSENDE'] = 0;
        $field_match_values['DUNNED_FUTURE_BOOKING_AMOUNT'] = 0;
        $field_match_values['ZINSSATZ_UEBER_BASISZINS'] = $plugin_billing_ueber_basiszinssatz;
        
        $field_match_values['RECHNUNGEN'] = array();
        foreach($forderungen as $key => $forderung_obj)
        {
            $forderung = $forderung_obj->_getForderung();
            $redetails = array();
            $redetails['NR'] = $forderung['NR'];
            $redetails['FaelligDatum'] = $forderung['FaelligDatum'];
            $redetails['Created'] = $forderung['Created'];
            $redetails['Price'] = $forderung['Price'];
            $redetails['Forderungstyp'] = $forderung['Forderungstyp'];
            $redetails['IncludedTax'] = $forderung['IncludedTax'];
            $redetails['PayType'] = $forderung['PayType'];


            $field_match_values['RECHNUNGEN'][] = $redetails;
        }

        $field_match_values['SUMME_OFFENE_RECHNUNGEN'] = $forderung_total;

        $post_data = array(
            'schuldnerid' => $new_schuldner_id,
        );
        $url = "https://".urlencode($this->username).":".urlencode($this->password)."@kundenverwaltung.mahn-rot.de/api/index.php?action=submit_forderungen";
        $post_data = array_merge($post_data, $field_match_values);
        $post_data = http_build_query($post_data);

        $context_options = array(
                'ssl'=>array(
                  'verify_peer'=>false,
                  'verify_peer_name'=>false,
                  'allow_self_signed'=>true,
                ),
                'http'=>
                array(
                    'timeout' => 5,
                )
              );
        $context_options['http'] = array (
                    'method' => 'POST',
                    'header'=> "Content-type: application/x-www-form-urlencoded\r\n",
                    'content' => $post_data
                    );
        $context = stream_context_create($context_options);
        $json_data = file_get_contents($url, false, $context);
        //var_dump($json_data);
        $retdata_billing_tmp = json_decode($json_data, true);
        if($retdata_billing_tmp['status'] != 'ok')
        {
            throw new Exception("ERROR: Fehler beim transfer (".$retdata_billing_tmp['msg'].")");
        }
       
        //dokumente zu den forderungen
        foreach($forderungen as $key => $forderung)
        {
            $docs = $forderung->_getDocs();
            foreach($docs as $key2 => $doc)
            {
                $this->_transferDocument($new_schuldner_id, $forderung, $doc, $key, $key2);
            }
        }
        
        return true;
    }
    
    private function _transferDocument($new_schuldner_id, $forderung_obj, $doc, $invoice_position, $doc_position)
    {
        $forderung = $forderung_obj->_getForderung();
        $metadata_send = $doc->_getMetadata();
        $metadata_send = array_merge($metadata_send, array(
            'uploaddocument' => 'yes',
            'invoiceposition' => $invoice_position,
            'invoice' => $forderung['NR'],
            'document_name' => $doc->getName(),
            'document_type' => $doc->getType(),
            'document_visibility' => $doc->getVisibility(),
            'document_descr' => $doc->getDescription(),
            'doc_position' => $doc_position,
        ));
        $post = array(
            'schuldnerid' => $new_schuldner_id,
            'description' => 'Dokument RENR '.$forderung['NR']." #".$doc_position." ".$doc->getName(),
            'fromdate' => date("Y-m-d"),
            'filename' => $new_schuldner_id.'_Dokument '.$doc->getName(),
            'action' => 'transfermahndokument',
            'uplfile'=> $doc->_getData());
            
            

        
        // Check data length
        $data_length = $doc->_getData() ? strlen($doc->_getData()) : 0;
       

        foreach($metadata_send as $mkey => $mval)
        {
            $post['metadata['.$mkey.']'] = $mval;
        }
        $ch = curl_init();
        $url = "https://".urlencode($this->username).":".urlencode($this->password)."@kundenverwaltung.mahn-rot.de/api/index.php?action=submit_document";
        curl_setopt($ch, CURLOPT_URL,$url);
        curl_setopt($ch, CURLOPT_POST,1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,0);
        error_log("[MAHN ROT API] cURL POST to: {$url}");
        error_log("[MAHN ROT API] Fields: " . print_r($post, true));
        $result = curl_exec ($ch);
        if($result === false)
        {
            //echo curl_error($ch);
            //echo "Fehler beim Dokumententransfer!";
            error_log("[MAHN ROT API] cURL error: " . curl_error($ch));
            throw new Exception("Fehler beim Dokumententransfer!");
            return false;
        }
        error_log("[MAHN ROT API] cURL transfer_document response: " . substr($result, 0, 500));
        curl_close ($ch);
        return true;
    }
}



class MahnRot_Akte
{
    private $schuldner;
    private $forderungen = array();
    
    public function __construct(MahnRot_Schuldner $schuldner)
    {
        $this->schuldner = $schuldner;
    }
    
    public function addForderung(MahnRot_Forderung $forderung)
    {
        $this->forderungen[] = $forderung;
    }
    
    public function getSchuldner()
    {
        return $this->schuldner;
    }
    
    public function getForderungen()
    {
        return $this->forderungen;
    }
}

class MahnRot_SchuldnerType
{
        public const PRIVATE='Privatkunde';
        public const COMPANY='Firma';
}

class MahnRot_Schuldner
{
    private $schuldnertype;
    private $schuldner_data = array();

    public function __construct($schuldnertype, $schuldner_data) // Removed "=MahnRot_SchuldnerType::PRIVATE"
    {
        $this->schuldnertype = $schuldnertype;
        $this->schuldner_data = $schuldner_data;
    }
    
    public function getSchuldnerType()
    {
        return $this->schuldnertype;
    }

    public function getSchuldnerData()
    {
        return $this->schuldner_data;
    }
}

class MahnRot_Forderung
{
    private $forderung = array();
    private $docs = array();
    
    public function __construct($invoice_number, $created_date, $due_date, $sum_cents, $sum_included_tax_cents, $foderung_paytype='HF_RE', $forderungs_typ='Warenlieferung')
    {
        $this->forderung = array(
            'NR' => $invoice_number,
            'FaelligDatum' => $due_date,
            'Created' => $created_date,
            'Status' => 'requested',
            'Price' => round($sum_cents / 100, 2),
            'Forderungstyp' => $forderungs_typ,
            'IncludedTax' => round($sum_included_tax_cents / 100, 2),
            'PayType' => $foderung_paytype,
        );
    }
    
    public function addDokument(MahnRot_Dokument $doc)
    {
        $this->docs[] = $doc;
    }
    
    public function _getDocs()
    {
        return $this->docs;
    }
    
    public function _getForderung()
    {
        return $this->forderung;
    }
    
    public function getTotalPrice($cents=true)
    {
        if($cents)
        {
            return $this->forderung['Price'] * 100;
        }
        return $this->forderung['Price'];
    }
}

class MahnRot_DokumentVisibility
{
        public const VISIBLE='visible';
        public const INVISIBLE='invisible';
}

class MahnRot_DokumentDocType
{
        public const Rechnungsdokument='Rechnungsdokument';
        public const Vertragsnachweis='Vertragsnachweis';
        public const Sonstiges='Sonstiges';
}

class MahnRot_Dokument
{
    private $data = false;
    private $metadata = array();
    
    private $name = "";
    private $type = false;
    private $visibility = "invisible";
    private $description = "";
    
    /*
     * 'invoice' => $invoice['Rechnungsnummer'],
                                        'document_name' => $dokument['name'],
                                        'document_type' => $dokument['type'], //<option>Rechnungsdokument</option>
                                  <option>Vertragsnachweis</option>
                                  <option>Sonstiges</option>
                                        'document_visibility' => $dokument['visibility'], //visible | invisible
                                        'document_descr' => $dokument['descr'],
     */
    public function __construct($doc_name, $doc_type, $doc_visibility, $doc_description)
    {
        $this->name = $doc_name;
        $this->type = $doc_type;
        $this->visibility = $doc_visibility;
        $this->description = $doc_description;
    }
    
    public function loadFromFile($filename)
    {
        $this->data = file_get_contents($filename);
        if($this->data === false)
        {
            return false;
        }
        return true;
    }
    
    public function setData($data)
    {
        $this->data = $data;
    }
    
    public function _getData()
    {
        return $this->data;
    }
    
    public function addMetadata($metadata, $metadata_val=false)
    {
        if(is_array($metadata))
        {
            $this->metadata = array_merge($this->metadata, $metadata);
            return true;
        }
        $this->metadata[$metadata] = $metadata_val;
        return true;
    }
    
    public function _getMetadata()
    {
        return $this->metadata;
    }
    
    public function getName()
    {
        return $this->name;
    }
    
    public function getType()
    {
        return $this->type;
    }
    
    public function getVisibility()
    {
        return $this->visibility;
    }
    
    public function getDescription()
    {
        return $this->description;
    }
}